<?php
/**
 * @file
 * Contains RobokassaPaymentController.
 */

/**
 * A Robokassa payment method controller.
 */
class RobokassaPaymentController extends PaymentMethodController {

  /**
   * Robokassa server payment URL.
   */
  const SERVER_URL = 'https://auth.robokassa.ru/Merchant/Index.aspx';
  const TEST_SERVER_URL = 'https://auth.robokassa.ru/Merchant/Index.aspx';

  /**
   * Default controller data.
   *
   * @var array
   */
  public $controller_data_defaults = array(
    'MrchLogin' => '',
    'pass1' => '',
    'pass2' => '',
    'server' => 'test',
    'hash_type' => 'md5',
  );

  /**
   * Payment method configuration form elements callback.
   *
   * @var string
   */
  public $payment_method_configuration_form_elements_callback = 'robokassa_payment_payment_method_configuration_form_elements';

  /**
   * Payment configuration form elements callback.
   *
   * @var string
   */
  public $payment_configuration_form_elements_callback = 'robokassa_payment_payment_configuration_form_elements';

  /**
   * RobokassaPaymentController constructor.
   */
  public function __construct() {
    $this->title = t('Robokassa');
    $this->description = t('Robokassa payment method.');
  }

  /**
   * Implements PaymentMethodController::execute().
   */
  public function execute(Payment $payment) {
    entity_save('payment', $payment);
    drupal_goto('robokassa/redirect/' . $payment->pid);
  }

  /**
   * Payment form generator.
   *
   * @param \Payment $payment
   *   Payment object.
   *
   * @return array
   *   Payment form array.
   */
  static public function getPaymentForm(Payment $payment) {
    $form = array();
    $form['#action'] = self::SERVER_URL;
    $settings = $payment->method->controller_data;

    // For test transactions.
    if ($settings['server'] == 'test') {
      $settings['IsTest'] = '1';
      $form['#action'] = self::TEST_SERVER_URL;
    }

    $form_data = array(
      'MrchLogin' => $settings['MrchLogin'],
      'InvId' => $payment->pid,
      'OutSum' => $payment->totalAmount(TRUE),
      'SignatureValue' => self::signature($payment),
    );
    foreach ($form_data as $key => $value) {
      $form[$key] = array(
        '#type' => 'hidden',
        '#value' => $value,
      );
    }
    return $form;
  }

  /**
   * Post data validator.
   *
   * @param array $data
   *   Post data.
   *
   * @return bool
   *   TRUE if post data is valid FALSE otherwise.
   */
  static public function validatePost(array $data, $is_interaction = TRUE) {
    // Exit now if the $_POST was empty.
    if (empty($data)) {
      watchdog('robokassa_payment', 'Interaction URL accessed with no POST data submitted.', array(), WATCHDOG_WARNING);
      return FALSE;
    }

    // Exit now if any required keys are not exists in $_POST.
    $required_keys = array('OutSum','InvId');
    if ($is_interaction) {
      $required_keys[] = 'SignatureValue';
    }
    $unavailable_required_keys = array_diff_key(array_flip($required_keys), $data);
    if (!empty($unavailable_required_keys)) {
      watchdog('robokassa_payment', 'Missing POST keys. POST data: <pre>!data</pre>', array('!data' => print_r($unavailable_required_keys, TRUE)), WATCHDOG_WARNING);
      return FALSE;
    }

    $payment = entity_load_single('payment', $data['InvId']);
    if (!$payment) {
      watchdog('robokassa_payment', 'Missing transaction id.  POST data: !data', array('!data' => print_r($data, TRUE)), WATCHDOG_WARNING);
      return FALSE;
    }

    $settings = $payment->method->controller_data;
    // Exit now if missing Checkout ID.
    if (empty($settings['MrchLogin'])) {
      $info = array(
        '!settings' => print_r($settings, 1),
        '!data' => print_r($data, TRUE),
      );
      watchdog('robokassa_payment !data', 'Missing merchant ID.  POST data: <pre>!data</pre> <pre>!settings</pre>',
        $info, WATCHDOG_WARNING);
      return FALSE;
    }


    if ($is_interaction) {
      if ($payment) {
        // Robokassa Signature.
        $robo_sign = $data['SignatureValue'];

        // Create own Signature.
        $signature_data = array(
          $data['OutSum'],
          $data['InvId'],
          $payment->method->controller_data['pass2'],
        );

        $sign = hash($settings['hash_type'], implode(':', $signature_data));

        // Exit now if missing Signature.
        if (drupal_strtoupper($robo_sign) != drupal_strtoupper($sign)) {
          watchdog('robokassa_payment', 'Missing Signature.  POST data: !data', array('!data' => print_r($data, TRUE)), WATCHDOG_WARNING);
          return FALSE;
        }
      }
    }

    if ($payment->totalAmount(TRUE) != $data['OutSum']) {
      watchdog('robokassa_payment', 'Missing transaction id amount.  POST data: !data', array('!data' => print_r($data, TRUE)), WATCHDOG_WARNING);
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Generate payment signature.
   *
   * @param \Payment $payment
   *   Payment object.
   *
   * @return string
   *   Signature.
   */
  static public function signature(Payment $payment) {
    $settings = $payment->method->controller_data;
    $signature_data = array(
      $settings['MrchLogin'],
      $payment->totalAmount(TRUE),
      $payment->pid,
      $settings['pass1'],
    );

    // Calculate signature.
    return hash($settings['hash_type'], implode(':', $signature_data));
  }

}
